#!/usr/bin/env python

# ident_scan.py
# Identd Tricker Scan
# by Joel R. Voss aka. Javantea
# Nov 28, 2005

"""
Identd is an interesting program. It searches /proc/net/tcp for a matching 
entry. The input is easy enough for a human to input into telnet if they have 
the localport and the remoteport. We get the localport from getsockname() and 
the remote port is the port that we are connected to. This will give us the 
user that is running the command.

Sadly this does not work on mIRC by default. mIRC only opens the Identd server 
during connection. A quick option flip gives us the proper result.
Received '43888, 113 : USERID : UNIX : Javantea\r\n'

It works mildly well with pidentd on Linux.
Received '51861 , 113 : USERID : OTHER :1001\r\n'
1001 is the first user created, which definitely is not enough information for 
any good use.

Although identd is a vulnerability, it is not a very big one considering how 
well default implementations keep the data private. The trick is in fact a flaw 
in the protocol. It allows any internet user to get a user id or name just by 
connecting.
"""

import socket

def identd_trick(host = '127.0.0.1', port = 113):
	s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	# connect to the identd server
	s.connect((host, port))
	# figure out our local port
	local_addr = s.getsockname()
	local_port = local_addr[1]
	# send local port, remote port
	s.send(str(local_port) + "," + str(port) + "\n")
	# get the response
	data = s.recv(1024)
	s.close()
	# print whatever it returned
	print 'Received', data
#end def identd_trick(host, port)

def main():
	from sys import argv
	if len(argv) > 1:
		HOST = argv[1]
	else:
		HOST = '127.0.0.1'    # The remote host
	#end if
	if len(argv) > 2:
		PORT = int(argv[2])
	else:
		PORT = 113            # The identd port
	#end if
	try:
		identd_trick(HOST, PORT)
	except Exception, ex:
		print ex
	#end try
#end def main()

if __name__ == '__main__':
	main()
#end if
